@@ -19,18 +19,18 @@
 
 /**
  * SECTION:element-a52dec
- * @title: a52dec
  *
  * Dolby Digital (AC-3) audio decoder.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 dvdreadsrc title=1 ! mpegpsdemux ! a52dec ! audioconvert ! audioresample ! autoaudiosink
  * ]| Play audio part of a dvd title.
  * |[
  * gst-launch-1.0 filesrc location=abc.ac3 ! ac3parse ! a52dec ! audioconvert ! audioresample ! autoaudiosink
  * ]| Decode and play a stand alone AC-3 file.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,10 +38,9 @@
 #endif
 
 #include <string.h>
+
 #include <stdlib.h>
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
+#include "_stdint.h"
 
 #include <gst/gst.h>
 
@@ -98,8 +97,8 @@ G_DEFINE_TYPE (GstA52Dec, gst_a52dec, GST_TYPE_AUDIO_DECODER);
 static gboolean gst_a52dec_start (GstAudioDecoder * dec);
 static gboolean gst_a52dec_stop (GstAudioDecoder * dec);
 static gboolean gst_a52dec_set_format (GstAudioDecoder * bdec, GstCaps * caps);
-static GstFlowReturn gst_a52dec_parse (GstAudioDecoder * dec,
-    GstAdapter * adapter, gint * offset, gint * length);
+static gboolean gst_a52dec_parse (GstAudioDecoder * dec, GstAdapter * adapter,
+    gint * offset, gint * length);
 static GstFlowReturn gst_a52dec_handle_frame (GstAudioDecoder * dec,
     GstBuffer * buffer);
 
@@ -186,10 +185,12 @@ gst_a52dec_class_init (GstA52DecClass * klass)
       g_param_spec_boolean ("lfe", "LFE", "LFE", TRUE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template (gstelement_class, &sink_factory);
-  gst_element_class_add_static_pad_template (gstelement_class, &src_factory);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
   gst_element_class_set_static_metadata (gstelement_class,
-      "ATSC A/52 audio decoder", "Codec/Decoder/Audio/Converter",
+      "ATSC A/52 audio decoder", "Codec/Decoder/Audio",
       "Decodes ATSC A/52 encoded audio streams",
       "David I. Lehn <dlehn@users.sourceforge.net>");
 
@@ -217,8 +218,6 @@ gst_a52dec_class_init (GstA52DecClass * klass)
 #endif
 
   GST_LOG ("CPU flags: a52=%08x, orc=%08x", klass->a52_cpuflags, cpuflags);
-
-  gst_type_mark_as_plugin_api (GST_TYPE_A52DEC_MODE, 0);
 }
 
 static void
@@ -230,10 +229,6 @@ gst_a52dec_init (GstA52Dec * a52dec)
   a52dec->state = NULL;
   a52dec->samples = NULL;
 
-  gst_audio_decoder_set_use_default_pad_acceptcaps (GST_AUDIO_DECODER_CAST
-      (a52dec), TRUE);
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_DECODER_SINK_PAD (a52dec));
-
   /* retrieve and intercept base class chain.
    * Quite HACKish, but that's dvd specs/caps for you,
    * since one buffer needs to be split into 2 frames */
@@ -627,7 +622,6 @@ gst_a52dec_handle_frame (GstAudioDecoder * bdec, GstBuffer * buffer)
             ("error decoding block %d", i), result);
         if (result != GST_FLOW_OK) {
           gst_buffer_unmap (outbuf, &map);
-          gst_buffer_unref (outbuf);
           goto exit;
         }
       } else {
