@@ -19,19 +19,19 @@
 
 /**
  * SECTION:element-amrnbenc
- * @title: amrnbenc
  * @see_also: #GstAmrnbDec, #GstAmrnbParse
  *
- * AMR narrowband encoder based on the
- * [opencore codec implementation](http://sourceforge.net/projects/opencore-amr).
- *
- * ## Example launch line
+ * AMR narrowband encoder based on the 
+ * <ulink url="http://sourceforge.net/projects/opencore-amr">opencore codec implementation</ulink>.
+ * 
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 filesrc location=abc.wav ! wavparse ! audioconvert ! audioresample ! amrnbenc ! filesink location=abc.amr
  * ]|
  * Please note that the above stream misses the header, that is needed to play
  * the stream.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -98,8 +98,6 @@ static GstFlowReturn gst_amrnbenc_handle_frame (GstAudioEncoder * enc,
 
 #define gst_amrnbenc_parent_class parent_class
 G_DEFINE_TYPE (GstAmrnbEnc, gst_amrnbenc, GST_TYPE_AUDIO_ENCODER);
-GST_ELEMENT_REGISTER_DEFINE (amrnbenc, "amrnbenc", GST_RANK_SECONDARY,
-    GST_TYPE_AMRNBENC);
 
 static void
 gst_amrnbenc_set_property (GObject * object, guint prop_id,
@@ -156,8 +154,10 @@ gst_amrnbenc_class_init (GstAmrnbEncClass * klass)
           BANDMODE_DEFAULT,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 
   gst_element_class_set_static_metadata (element_class, "AMR-NB audio encoder",
       "Codec/Encoder/Audio",
@@ -166,14 +166,11 @@ gst_amrnbenc_class_init (GstAmrnbEncClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (gst_amrnbenc_debug, "amrnbenc", 0,
       "AMR-NB audio encoder");
-
-  gst_type_mark_as_plugin_api (GST_AMRNBENC_BANDMODE_TYPE, 0);
 }
 
 static void
 gst_amrnbenc_init (GstAmrnbEnc * amrnbenc)
 {
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_ENCODER_SINK_PAD (amrnbenc));
 }
 
 static gboolean
