@@ -19,17 +19,17 @@
 
 /**
  * SECTION:element-amrwbdec
- * @title: amrwbdec
  * @see_also: #GstAmrwbEnc
  *
- * AMR wideband decoder based on the
- * [opencore codec implementation](http://sourceforge.net/projects/opencore-amr).
+ * AMR wideband decoder based on the 
+ * <ulink url="http://sourceforge.net/projects/opencore-amr">opencore codec implementation</ulink>.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 filesrc location=abc.amr ! amrparse ! amrwbdec ! audioconvert ! audioresample ! autoaudiosink
  * ]|
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -69,8 +69,8 @@ static const unsigned char block_size[16] =
 static gboolean gst_amrwbdec_start (GstAudioDecoder * dec);
 static gboolean gst_amrwbdec_stop (GstAudioDecoder * dec);
 static gboolean gst_amrwbdec_set_format (GstAudioDecoder * dec, GstCaps * caps);
-static GstFlowReturn gst_amrwbdec_parse (GstAudioDecoder * dec,
-    GstAdapter * adapter, gint * offset, gint * length);
+static gboolean gst_amrwbdec_parse (GstAudioDecoder * dec, GstAdapter * adapter,
+    gint * offset, gint * length);
 static GstFlowReturn gst_amrwbdec_handle_frame (GstAudioDecoder * dec,
     GstBuffer * buffer);
 
@@ -83,8 +83,10 @@ gst_amrwbdec_class_init (GstAmrwbDecClass * klass)
   GstAudioDecoderClass *base_class = GST_AUDIO_DECODER_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 
   gst_element_class_set_static_metadata (element_class, "AMR-WB audio decoder",
       "Codec/Decoder/Audio",
@@ -105,9 +107,6 @@ static void
 gst_amrwbdec_init (GstAmrwbDec * amrwbdec)
 {
   gst_audio_decoder_set_needs_format (GST_AUDIO_DECODER (amrwbdec), TRUE);
-  gst_audio_decoder_set_use_default_pad_acceptcaps (GST_AUDIO_DECODER_CAST
-      (amrwbdec), TRUE);
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_DECODER_SINK_PAD (amrwbdec));
 }
 
 static gboolean
@@ -172,12 +171,14 @@ gst_amrwbdec_parse (GstAudioDecoder * dec, GstAdapter * adapter,
   gint block, mode;
 
   size = gst_adapter_available (adapter);
-  if (size < 1)
-    return GST_FLOW_ERROR;
+  g_return_val_if_fail (size > 0, GST_FLOW_ERROR);
 
   gst_audio_decoder_get_parse_state (dec, &sync, &eos);
 
   /* need to peek data to get the size */
+  if (size < 1)
+    return GST_FLOW_ERROR;
+
   gst_adapter_copy (adapter, header, 0, 1);
   mode = (header[0] >> 3) & 0x0F;
   block = block_size[mode];
