@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -41,19 +41,19 @@
  * <refsect2>
  * <title>Example pipelines</title>
  * |[
- * gst-launch -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! lamemp3enc ! filesink location=sine.mp3
+ * gst-launch-1.0 -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! lamemp3enc ! filesink location=sine.mp3
  * ]| Encode a test sine signal to MP3.
  * |[
- * gst-launch -v alsasrc ! audioconvert ! lamemp3enc target=bitrate bitrate=192 ! filesink location=alsasrc.mp3
+ * gst-launch-1.0 -v autoaudiosrc ! audioconvert ! lamemp3enc target=bitrate bitrate=192 ! filesink location=alsasrc.mp3
  * ]| Record from a sound card using ALSA and encode to MP3 with an average bitrate of 192kbps
  * |[
- * gst-launch -v filesrc location=music.wav ! decodebin ! audioconvert ! audioresample ! lamemp3enc target=quality quality=0 ! id3v2mux ! filesink location=music.mp3
+ * gst-launch-1.0 -v filesrc location=music.wav ! decodebin ! audioconvert ! audioresample ! lamemp3enc target=quality quality=0 ! id3v2mux ! filesink location=music.mp3
  * ]| Transcode from a .wav file to MP3 (the id3v2mux element is optional) with best VBR quality
  * |[
- * gst-launch -v cdda://5 ! audioconvert ! lamemp3enc target=bitrate cbr=true bitrate=192 ! filesink location=track5.mp3
+ * gst-launch-1.0 -v cdda://5 ! audioconvert ! lamemp3enc target=bitrate cbr=true bitrate=192 ! filesink location=track5.mp3
  * ]| Encode Audio CD track 5 to MP3 with a constant bitrate of 192kbps
  * |[
- * gst-launch -v audiotestsrc num-buffers=10 ! audio/x-raw,rate=44100,channels=1 ! lamemp3enc target=bitrate cbr=true bitrate=48 ! filesink location=test.mp3
+ * gst-launch-1.0 -v audiotestsrc num-buffers=10 ! audio/x-raw,rate=44100,channels=1 ! lamemp3enc target=bitrate cbr=true bitrate=48 ! filesink location=test.mp3
  * ]| Encode to a fixed sample rate
  * </refsect2>
  *
@@ -119,7 +119,7 @@ static GType
 gst_lamemp3enc_target_get_type (void)
 {
   static GType lame_target_type = 0;
-  static GEnumValue lame_targets[] = {
+  static const GEnumValue lame_targets[] = {
     {LAMEMP3ENC_TARGET_QUALITY, "Quality", "quality"},
     {LAMEMP3ENC_TARGET_BITRATE, "Bitrate", "bitrate"},
     {0, NULL, NULL}
@@ -144,7 +144,7 @@ static GType
 gst_lamemp3enc_encoding_engine_quality_get_type (void)
 {
   static GType lame_encoding_engine_quality_type = 0;
-  static GEnumValue lame_encoding_engine_quality[] = {
+  static const GEnumValue lame_encoding_engine_quality[] = {
     {0, "Fast", "fast"},
     {1, "Standard", "standard"},
     {2, "High", "high"},
@@ -653,6 +653,7 @@ gst_lamemp3enc_finish_frames (GstLameMP3Enc * lame)
     guint rate, version, layer, size;
     GstBuffer *mp3_buf;
     const guint8 *data;
+    guint samples_per_frame;
 
     data = gst_adapter_map (lame->adapter, 4);
     header = GST_READ_UINT32_BE (data);
@@ -677,11 +678,18 @@ gst_lamemp3enc_finish_frames (GstLameMP3Enc * lame)
       break;
     }
 
+    /* Account for the internal resampling, finish frame really wants to
+     * know about the number of incoming samples
+     */
+    samples_per_frame = (version == 1) ? 1152 : 576;
+    samples_per_frame *= lame->samplerate;
+    samples_per_frame /= lame->out_samplerate;
+
     /* should be ok now */
     mp3_buf = gst_adapter_take_buffer (lame->adapter, size);
     /* number of samples for MPEG-1, layer 3 */
     result = gst_audio_encoder_finish_frame (GST_AUDIO_ENCODER (lame),
-        mp3_buf, version == 1 ? 1152 : 576);
+        mp3_buf, samples_per_frame);
   }
 
 exit:
