@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <unistd.h>
@@ -30,7 +30,7 @@
 static GstPad *mysrcpad, *mysinkpad;
 
 #define VIDEO_CAPS_STRING "video/x-raw, " \
-                           "format = (string) I420, " \
+                           "format = (string) { I420, Y42B, Y444 }, " \
                            "width = (int) 384, " \
                            "height = (int) 288, " \
                            "framerate = (fraction) 25/1"
@@ -39,30 +39,42 @@ static GstPad *mysrcpad, *mysinkpad;
                            "width = (int) 384, " \
                            "height = (int) 288, " \
                            "framerate = (fraction) 25/1"
-
-static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (MPEG_CAPS_STRING));
-
-static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
-    GST_PAD_SRC,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (VIDEO_CAPS_STRING));
-
-
 static GstElement *
-setup_x264enc (void)
+setup_x264enc (const gchar * profile, const gchar * stream_format,
+    const gchar * input_format)
 {
+  GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
+      GST_PAD_SINK,
+      GST_PAD_ALWAYS,
+      GST_STATIC_CAPS (MPEG_CAPS_STRING));
+
+  GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
+      GST_PAD_SRC,
+      GST_PAD_ALWAYS,
+      GST_STATIC_CAPS (VIDEO_CAPS_STRING));
   GstElement *x264enc;
+  gchar *caps_str;
+  GstCaps *caps;
 
   GST_DEBUG ("setup_x264enc");
+
+  caps_str = g_strdup_printf ("%s, profile = (string) %s, "
+      "stream-format = (string) %s", MPEG_CAPS_STRING, profile, stream_format);
+  sinktemplate.static_caps.string = caps_str;
+
   x264enc = gst_check_setup_element ("x264enc");
   mysrcpad = gst_check_setup_src_pad (x264enc, &srctemplate);
   mysinkpad = gst_check_setup_sink_pad (x264enc, &sinktemplate);
   gst_pad_set_active (mysrcpad, TRUE);
   gst_pad_set_active (mysinkpad, TRUE);
 
+  caps = gst_caps_from_string (VIDEO_CAPS_STRING);
+  gst_caps_set_simple (caps, "format", G_TYPE_STRING, input_format, NULL);
+  gst_check_setup_events (mysrcpad, x264enc, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+
+  g_free (caps_str);
+
   return x264enc;
 }
 
@@ -80,11 +92,12 @@ cleanup_x264enc (GstElement * x264enc)
 }
 
 static void
-check_caps (GstCaps * caps, gint profile_id)
+check_caps (GstCaps * caps, const gchar * profile, gint profile_id)
 {
   GstStructure *s;
-  const GValue *sf, *avcc;
+  const GValue *sf, *avcc, *pf;
   const gchar *stream_format;
+  const gchar *caps_profile;
 
   fail_unless (caps != NULL);
 
@@ -115,10 +128,18 @@ check_caps (GstCaps * caps, gint profile_id)
   } else {
     fail_if (TRUE, "unexpected stream-format in caps: %s", stream_format);
   }
+
+  pf = gst_structure_get_value (s, "profile");
+  fail_unless (pf != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (pf));
+  caps_profile = g_value_get_string (pf);
+  fail_unless (caps_profile != NULL);
+  fail_unless (!strcmp (caps_profile, profile));
 }
 
 static void
-test_video_profile (const gchar * profile, gint profile_id)
+test_video_profile (const gchar * profile, gint profile_id,
+    const gchar * input_format)
 {
   GstElement *x264enc;
   GstBuffer *inbuffer, *outbuffer;
@@ -128,25 +149,19 @@ test_video_profile (const gchar * profile, gint profile_id)
   gsize size;
   int i, num_buffers;
 
-  x264enc = setup_x264enc ();
+  x264enc = setup_x264enc (profile, "avc", input_format);
   fail_unless (gst_element_set_state (x264enc,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
 
-  caps = gst_caps_from_string (MPEG_CAPS_STRING);
-  gst_caps_set_simple (caps, "profile", G_TYPE_STRING, profile, NULL);
-  /* code below assumes avc */
-  gst_caps_set_simple (caps, "stream-format", G_TYPE_STRING, "avc", NULL);
-  gst_pad_set_caps (mysinkpad, caps);
-  gst_caps_unref (caps);
-  gst_pad_use_fixed_caps (mysinkpad);
-
-  caps = gst_caps_from_string (VIDEO_CAPS_STRING);
-  fail_unless (gst_pad_set_caps (mysrcpad, caps));
-  gst_caps_unref (caps);
-
   /* corresponds to I420 buffer for the size mentioned in the caps */
-  inbuffer = gst_buffer_new_and_alloc (384 * 288 * 3 / 2);
+  if (!strcmp (input_format, "I420"))
+    inbuffer = gst_buffer_new_and_alloc (384 * 288 * 3 / 2);
+  else if (!strcmp (input_format, "Y42B"))
+    inbuffer = gst_buffer_new_and_alloc (384 * 288 * 2);
+  else if (!strcmp (input_format, "Y444"))
+    inbuffer = gst_buffer_new_and_alloc (384 * 288 * 3);
+
   /* makes valgrind's memcheck happier */
   gst_buffer_memset (inbuffer, 0, 0, -1);
   GST_BUFFER_TIMESTAMP (inbuffer) = 0;
@@ -164,7 +179,7 @@ test_video_profile (const gchar * profile, gint profile_id)
     GstCaps *outcaps;
 
     outcaps = gst_pad_get_current_caps (mysinkpad);
-    check_caps (outcaps, profile_id);
+    check_caps (outcaps, profile, profile_id);
     gst_caps_unref (outcaps);
   }
 
@@ -239,25 +254,41 @@ test_video_profile (const gchar * profile, gint profile_id)
 
 GST_START_TEST (test_video_baseline)
 {
-  test_video_profile ("constrained-baseline", 0x42);
+  test_video_profile ("constrained-baseline", 0x42, "I420");
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_video_main)
 {
-  test_video_profile ("main", 0x4d);
+  test_video_profile ("main", 0x4d, "I420");
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_video_high)
 {
-  test_video_profile ("high", 0x64);
+  test_video_profile ("high", 0x64, "I420");
 }
 
 GST_END_TEST;
 
+GST_START_TEST (test_video_high422)
+{
+  test_video_profile ("high-4:2:2", 0x7A, "Y42B");
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_video_high444)
+{
+  test_video_profile ("high-4:4:4", 0xF4, "Y444");
+}
+
+GST_END_TEST;
+
+
+
 Suite *
 x264enc_suite (void)
 {
@@ -268,23 +299,10 @@ x264enc_suite (void)
   tcase_add_test (tc_chain, test_video_baseline);
   tcase_add_test (tc_chain, test_video_main);
   tcase_add_test (tc_chain, test_video_high);
+  tcase_add_test (tc_chain, test_video_high422);
+  tcase_add_test (tc_chain, test_video_high444);
 
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = x264enc_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (x264enc);
